@@ -390,7 +390,7 @@ function _insert_configs_from_envs(configs, envs, opt)
     opt = opt or {}
     local configs_str = opt._configs_str
     for k, v in pairs(envs) do
-        if configs_str and configs_str:find(k, 1, true) then
+        if configs_str and configs_str:find("-D" .. k .. "=", 1, true) then
             -- use user custom configuration
         else
             table.insert(configs, "-D" .. k .. "=" .. v)
@@ -419,8 +419,9 @@ function _get_configs_for_generic(package, configs, opt)
     local shflags = _get_shflags(package, opt)
     if shflags then
         table.insert(configs, "-DCMAKE_SHARED_LINKER_FLAGS=" .. shflags)
+        table.insert(configs, "-DCMAKE_MODULE_LINKER_FLAGS=" .. shflags)
     end
-    if package:config("pic") ~= false then
+    if not package:is_plat("windows", "mingw") and package:config("pic") ~= false then
         table.insert(configs, "-DCMAKE_POSITION_INDEPENDENT_CODE=ON")
     end
     if not package:use_external_includes() then
@@ -470,10 +471,20 @@ function _get_configs_for_windows(package, configs, opt)
     elseif package:has_runtime("MDd") then
         table.insert(configs, "-DCMAKE_MSVC_RUNTIME_LIBRARY=MultiThreadedDebugDLL")
     end
-    if not opt._configs_str:find("CMAKE_COMPILE_PDB_OUTPUT_DIRECTORY") then
-        table.insert(configs, "-DCMAKE_COMPILE_PDB_OUTPUT_DIRECTORY=pdb")
+
+    local pdb_dir = path.unix(path.join(os.curdir(), "pdb"))
+    if not opt._configs_str:find("CMAKE_COMPILE_PDB_OUTPUT_DIRECTORY", 1, true) then
+        table.insert(configs, "-DCMAKE_COMPILE_PDB_OUTPUT_DIRECTORY=" .. pdb_dir)
+    end
+    if not opt._configs_str:find("CMAKE_PDB_OUTPUT_DIRECTORY", 1, true) then
+        table.insert(configs, "-DCMAKE_PDB_OUTPUT_DIRECTORY=" .. pdb_dir)
+    end
+
+    if package:is_cross() then
+        _get_configs_for_cross(package, configs, opt)
+    else
+        _get_configs_for_generic(package, configs, opt)
     end
-    _get_configs_for_generic(package, configs, opt)
 end
 
 -- get configs for android
@@ -523,6 +534,7 @@ function _get_configs_for_appleos(package, configs, opt)
     envs.CMAKE_STATIC_LINKER_FLAGS = table.concat(table.wrap(package:build_getenv("arflags")), ' ')
     envs.CMAKE_EXE_LINKER_FLAGS    = _get_ldflags(package, opt)
     envs.CMAKE_SHARED_LINKER_FLAGS = _get_shflags(package, opt)
+    envs.CMAKE_MODULE_LINKER_FLAGS = _get_shflags(package, opt)
     -- https://cmake.org/cmake/help/v3.17/manual/cmake-toolchains.7.html#id25
     if package:is_plat("watchos") then
         envs.CMAKE_SYSTEM_NAME = "watchOS"
@@ -567,7 +579,12 @@ function _get_configs_for_mingw(package, configs, opt)
     envs.CMAKE_STATIC_LINKER_FLAGS = table.concat(table.wrap(package:build_getenv("arflags")), ' ')
     envs.CMAKE_EXE_LINKER_FLAGS    = _get_ldflags(package, opt)
     envs.CMAKE_SHARED_LINKER_FLAGS = _get_shflags(package, opt)
-    envs.CMAKE_SYSTEM_NAME         = "Windows"
+    envs.CMAKE_MODULE_LINKER_FLAGS = _get_shflags(package, opt)
+    -- @see https://cmake.org/cmake/help/latest/variable/CMAKE_CROSSCOMPILING.html
+    -- https://github.com/xmake-io/xmake/pull/5888
+    if not is_host("windows") then
+        envs.CMAKE_SYSTEM_NAME = "Windows"
+    end
     envs.CMAKE_SYSTEM_PROCESSOR    = _get_cmake_system_processor(package)
     -- avoid find and add system include/library path
     -- @see https://github.com/xmake-io/xmake/issues/2037
@@ -584,9 +601,6 @@ function _get_configs_for_mingw(package, configs, opt)
     if is_subhost("windows") and opt.cmake_generator ~= "Ninja" then
         envs.CMAKE_MAKE_PROGRAM = _get_mingw32_make(package)
     end
-    if opt.cmake_generator == "Ninja" then
-        envs.CMAKE_MAKE_PROGRAM = "ninja"
-    end
     _fix_cxx_compiler_cmake(package, envs)
     _insert_configs_from_envs(configs, envs, opt)
 end
@@ -601,12 +615,7 @@ function _get_configs_for_wasm(package, configs, opt)
     assert(emscripten_cmakefile, "Emscripten.cmake not found!")
     table.insert(configs, "-DCMAKE_TOOLCHAIN_FILE=" .. emscripten_cmakefile)
     if is_subhost("windows") then
-        if opt.cmake_generator == "Ninja" then
-            local ninja = _get_ninja(package)
-            if ninja then
-                table.insert(configs, "-DCMAKE_MAKE_PROGRAM=" .. ninja)
-            end
-        else
+        if opt.cmake_generator ~= "Ninja" then
             local mingw_make = _get_mingw32_make(package)
             if mingw_make then
                 table.insert(configs, "-DCMAKE_MAKE_PROGRAM=" .. mingw_make)
@@ -635,6 +644,9 @@ function _get_configs_for_cross(package, configs, opt)
     envs.CMAKE_CXX_COMPILER        = _translate_bin_path(package:build_getenv("cxx"))
     envs.CMAKE_ASM_COMPILER        = _translate_bin_path(package:build_getenv("as"))
     envs.CMAKE_AR                  = _translate_bin_path(package:build_getenv("ar"))
+    if package:is_plat("windows") and package:has_tool("cxx", "cl") then
+        envs.CMAKE_AR = path.join(path.directory(envs.CMAKE_CXX_COMPILER), "lib.exe")
+    end
     _fix_cxx_compiler_cmake(package, envs)
     -- @note The link command line is set in Modules/CMake{C,CXX,Fortran}Information.cmake and defaults to using the compiler, not CMAKE_LINKER,
     -- so we need to set CMAKE_CXX_LINK_EXECUTABLE to use CMAKE_LINKER as linker.
@@ -653,18 +665,21 @@ function _get_configs_for_cross(package, configs, opt)
     envs.CMAKE_STATIC_LINKER_FLAGS = table.concat(table.wrap(package:build_getenv("arflags")), ' ')
     envs.CMAKE_EXE_LINKER_FLAGS    = _get_ldflags(package, opt)
     envs.CMAKE_SHARED_LINKER_FLAGS = _get_shflags(package, opt)
+    envs.CMAKE_MODULE_LINKER_FLAGS = _get_shflags(package, opt)
     -- we don't need to set it as cross compilation if we just pass toolchain
     -- https://github.com/xmake-io/xmake/issues/2170
     if package:is_cross() then
         local system_name = package:targetos() or "Linux"
         if system_name == "linux" then
             system_name = "Linux"
+        elseif system_name == "windows" then
+            system_name = "Windows"
         end
         envs.CMAKE_SYSTEM_NAME = system_name
-    else
-        if package:config("pic") ~= false then
-            table.insert(configs, "-DCMAKE_POSITION_INDEPENDENT_CODE=ON")
-        end
+        envs.CMAKE_SYSTEM_PROCESSOR = _get_cmake_system_processor(package)
+    end
+    if not package:is_plat("windows", "mingw") and package:config("pic") ~= false then
+        table.insert(configs, "-DCMAKE_POSITION_INDEPENDENT_CODE=ON")
     end
     -- avoid find and add system include/library path
     -- @see https://github.com/xmake-io/xmake/issues/2037
@@ -712,14 +727,14 @@ function _get_configs_for_host_toolchain(package, configs, opt)
     envs.CMAKE_STATIC_LINKER_FLAGS = table.concat(table.wrap(package:build_getenv("arflags")), ' ')
     envs.CMAKE_EXE_LINKER_FLAGS    = _get_ldflags(package, opt)
     envs.CMAKE_SHARED_LINKER_FLAGS = _get_shflags(package, opt)
+    envs.CMAKE_MODULE_LINKER_FLAGS = _get_shflags(package, opt)
     -- we don't need to set it as cross compilation if we just pass toolchain
     -- https://github.com/xmake-io/xmake/issues/2170
     if package:is_cross() then
         envs.CMAKE_SYSTEM_NAME     = "Linux"
-    else
-        if package:config("pic") ~= false then
-            table.insert(configs, "-DCMAKE_POSITION_INDEPENDENT_CODE=ON")
-        end
+    end
+    if not package:is_plat("windows", "mingw") and package:config("pic") ~= false then
+        table.insert(configs, "-DCMAKE_POSITION_INDEPENDENT_CODE=ON")
     end
     _insert_configs_from_envs(configs, envs, opt)
 end
@@ -761,6 +776,10 @@ function _get_configs_for_generator(package, configs, opt)
                 table.insert(configs, "-DCMAKE_JOB_POOL_LINK:STRING=link")
                 table.insert(configs, ("-DCMAKE_JOB_POOLS:STRING=compile=%s;link=%s"):format(jobs, linkjobs))
             end
+            local ninja = _get_ninja(package)
+            if ninja then
+                table.insert(configs, "-DCMAKE_MAKE_PROGRAM=" .. ninja)
+            end
         end
     elseif package:is_plat("mingw") and is_subhost("msys") then
         table.insert(configs, "-G")
@@ -800,7 +819,12 @@ function _get_default_flags(package, configs, buildtype, opt)
         local tmpdir = path.join(os.tmpfile() .. ".dir", package:displayname(), package:mode())
         local dummy_cmakelist = path.join(tmpdir, "CMakeLists.txt")
 
+        -- About the minimum cmake version requirement
+        -- @see https://github.com/xmake-io/xmake/pull/6032
         io.writefile(dummy_cmakelist, format([[
+    cmake_minimum_required(VERSION 3.15)
+    project(XMakeDummyProject)
+
     message(STATUS "CMAKE_C_FLAGS is ${CMAKE_C_FLAGS}")
     message(STATUS "CMAKE_C_FLAGS_%s is ${CMAKE_C_FLAGS_%s}")
 
@@ -886,6 +910,7 @@ function _get_envs_for_runtime_flags(package, configs, opt)
         envs[format("CMAKE_EXE_LINKER_FLAGS_%s", buildtype)]    = toolchain_utils.map_linkflags_for_package(package, "binary", {"cxx"}, "runtime", runtimes)
         envs[format("CMAKE_STATIC_LINKER_FLAGS_%s", buildtype)] = toolchain_utils.map_linkflags_for_package(package, "static", {"cxx"}, "runtime", runtimes)
         envs[format("CMAKE_SHARED_LINKER_FLAGS_%s", buildtype)] = toolchain_utils.map_linkflags_for_package(package, "shared", {"cxx"}, "runtime", runtimes)
+        envs[format("CMAKE_MODULE_LINKER_FLAGS_%s", buildtype)] = toolchain_utils.map_linkflags_for_package(package, "shared", {"cxx"}, "runtime", runtimes)
     end
     return envs
 end
@@ -920,6 +945,13 @@ function _get_configs(package, configs, opt)
     else
         _get_configs_for_generic(package, configs, opt)
     end
+
+    -- fix error for cmake 4.x
+    -- e.g. Compatibility with CMake < 3.5 has been removed from CMake.
+    if _get_cmake_version() and _get_cmake_version():ge("4.0") then
+        table.insert(configs, "-DCMAKE_POLICY_VERSION_MINIMUM=3.5")
+    end
+
     local envs = _get_envs_for_default_flags(package, configs, opt)
     local runtime_envs = _get_envs_for_runtime_flags(package, configs, opt)
     if runtime_envs then
@@ -929,6 +961,13 @@ function _get_configs(package, configs, opt)
         end
     end
     _insert_configs_from_envs(configs, envs or {}, opt)
+
+    local ccache = package:data("ccache")
+    if ccache then
+        table.insert(configs, "-DCMAKE_C_COMPILER_LAUNCHER=" .. ccache)
+        table.insert(configs, "-DCMAKE_CXX_COMPILER_LAUNCHER=" .. ccache)
+    end
+
     return configs
 end
 
@@ -1064,7 +1103,7 @@ function _build_for_cmakebuild(package, configs, opt)
         table.insert(argv, "--target")
         if #targets > 1 then
             -- https://stackoverflow.com/questions/47553569/how-can-i-build-multiple-targets-using-cmake-build
-            if _get_cmake_version():ge("3.15") then
+            if _get_cmake_version() and _get_cmake_version():ge("3.15") then
                 table.join2(argv, targets)
             else
                 raise("Build multiple targets need cmake >=3.15")
@@ -1168,7 +1207,8 @@ function _get_cmake_generator(package, opt)
         if not cmake_generator then
             if package:has_tool("cc", "clang_cl") or package:has_tool("cxx", "clang_cl") then
                 cmake_generator = "Ninja"
-            elseif is_subhost("windows") and package:is_plat("mingw", "wasm") then
+            elseif (is_subhost("windows") and package:is_plat("mingw", "wasm"))
+                or (package:is_plat("windows") and is_host("linux")) then
                 local ninja = _get_ninja(package)
                 if ninja then
                     cmake_generator = "Ninja"
@@ -1186,6 +1226,80 @@ function _get_cmake_generator(package, opt)
     return cmake_generator
 end
 
+-- shrink cmake arguments, fix too long arguments
+-- @see https://github.com/xmake-io/xmake-repo/pull/5247#discussion_r1780302212
+function _shrink_cmake_arguments(argv, oldir, opt)
+    local cmake_argv = {}
+    local long_options = hashset.of(
+        "CMAKE_C_FLAGS",
+        "CMAKE_CXX_FLAGS",
+        "CMAKE_ASM_FLAGS",
+        "CMAKE_EXE_LINKER_FLAGS",
+        "CMAKE_SHARED_LINKER_FLAGS",
+        "CMAKE_MODULE_LINKER_FLAGS",
+        "CMAKE_C_FLAGS_RELEASE",
+        "CMAKE_CXX_FLAGS_RELEASE",
+        "CMAKE_ASM_FLAGS_RELEASE",
+        "CMAKE_EXE_LINKER_FLAGS_RELEASE",
+        "CMAKE_SHARED_LINKER_FLAGS_RELEASE",
+        "CMAKE_MODULE_LINKER_FLAGS_RELEASE",
+        "CMAKE_C_FLAGS_DEBUG",
+        "CMAKE_CXX_FLAGS_DEBUG",
+        "CMAKE_ASM_FLAGS_DEBUG",
+        "CMAKE_EXE_LINKER_FLAGS_DEBUG",
+        "CMAKE_SHARED_LINKER_FLAGS_DEBUG",
+        "CMAKE_MODULE_LINKER_FLAGS_DEBUG")
+    local shrink = false
+    local add_compile_options = false
+    local add_link_options = false
+    if _get_cmake_version() and _get_cmake_version():ge("3.13") then
+        add_compile_options = true
+        add_link_options = true
+    end
+    local buildtypes_map = {
+        RELEASE = "Release",
+        DEBUG = "Debug",
+        RELWITHDEBINFO = "RelWithDebInfo"
+    }
+    table.remove_if(argv, function (idx, value)
+        local k, v = value:match("%-D(.*)=(.*)")
+        if k and v and long_options:has(k) then
+            local kind, mode = k:match("CMAKE_(.+)_FLAGS_(.+)")
+            if not kind then
+                kind = k:match("CMAKE_(.+)_FLAGS")
+            end
+            -- improve cmake flags
+            -- @see https://github.com/xmake-io/xmake/issues/5826
+            --[[
+            local build_type = mode and buildtypes_map[mode] or nil
+            if #v > 0 and add_compile_options and (kind == "C" or kind == "CXX" or kind == "ASM") then
+                if build_type then
+                    table.insert(cmake_argv, ("if(CMAKE_BUILD_TYPE STREQUAL \"%s\")"):format(build_type))
+                end
+                local flags = v:replace("\"", "\\\"")
+                table.insert(cmake_argv, ("set(COMP_%s_FLAGS \"%s\")"):format(kind, flags))
+                table.insert(cmake_argv, ("add_compile_options($<$<COMPILE_LANGUAGE:%s>:${COMP_%s_FLAGS}>)"):format(kind, kind))
+                if build_type then
+                    table.insert(cmake_argv, "endif()")
+                end
+                shrink = true
+                return true
+            end]]
+            -- shrink long arguments
+            if #v > 128 then
+                local flags = v:replace("\"", "\\\"")
+                table.insert(cmake_argv, ("set(%s \"%s\")"):format(k, flags))
+                shrink = true
+                return true
+            end
+        end
+    end)
+    if shrink then
+        local cmakefile = path.join(opt.curdir and opt.curdir or oldir, "CMakeLists.txt")
+        io.insert(cmakefile, 1, table.concat(cmake_argv, "\n"))
+    end
+end
+
 function configure(package, configs, opt)
     opt = opt or {}
     local oldir = _enter_buildir(package, opt)
@@ -1204,36 +1318,7 @@ function configure(package, configs, opt)
     end
     -- shrink cmake arguments, fix too long arguments
     -- @see https://github.com/xmake-io/xmake-repo/pull/5247#discussion_r1780302212
-    local cmake_argv = {}
-    local long_options = hashset.of(
-        "CMAKE_C_FLAGS",
-        "CMAKE_CXX_FLAGS",
-        "CMAKE_ASM_FLAGS",
-        "CMAKE_EXE_LINKER_FLAGS",
-        "CMAKE_SHARED_LINKER_FLAGS",
-        "CMAKE_C_FLAGS_RELEASE",
-        "CMAKE_CXX_FLAGS_RELEASE",
-        "CMAKE_ASM_FLAGS_RELEASE",
-        "CMAKE_EXE_LINKER_FLAGS_RELEASE",
-        "CMAKE_SHARED_LINKER_FLAGS_RELEASE",
-        "CMAKE_C_FLAGS_DEBUG",
-        "CMAKE_CXX_FLAGS_DEBUG",
-        "CMAKE_ASM_FLAGS_DEBUG",
-        "CMAKE_EXE_LINKER_FLAGS_DEBUG",
-        "CMAKE_SHARED_LINKER_FLAGS_DEBUG")
-    local shrink = false
-    table.remove_if(argv, function (idx, value)
-        local k, v = value:match("%-D(.*)=(.*)")
-        if k and v and long_options:has(k) and #v > 128 then
-            table.insert(cmake_argv, ("set(%s \"%s\")"):format(k, tostring(v)))
-            shrink = true
-            return true
-        end
-    end)
-    if shrink then
-        local cmakefile = path.join(opt.curdir and opt.curdir or oldir, "CMakeLists.txt")
-        io.insert(cmakefile, 1, table.concat(cmake_argv, "\n"))
-    end
+    _shrink_cmake_arguments(argv, oldir, opt)
     table.insert(argv, oldir)
 
     -- do configure
