@@ -183,14 +183,14 @@ function _install_target_files(target, batchcmds_, opt)
     local srcfiles, dstfiles = target:installfiles(_get_target_installdir(package, target))
     if srcfiles and dstfiles then
         for idx, srcfile in ipairs(srcfiles) do
-            batchcmds_:cp(srcfile, dstfiles[idx])
+            batchcmds_:cp(srcfile, dstfiles[idx], {symlink = true})
         end
     end
     for _, dep in ipairs(target:orderdeps()) do
         local srcfiles, dstfiles = dep:installfiles(_get_target_installdir(package, dep), {interface = true})
         if srcfiles and dstfiles then
             for idx, srcfile in ipairs(srcfiles) do
-                batchcmds_:cp(srcfile, dstfiles[idx])
+                batchcmds_:cp(srcfile, dstfiles[idx], {symlink = true})
             end
         end
     end
@@ -227,7 +227,6 @@ function _install_target_shared_libraries(target, batchcmds_, opt)
 
     -- do install
     for _, libfile in ipairs(libfiles) do
-        local filename = path.filename(libfile)
         _copy_file_with_symlinks(batchcmds_, libfile, bindir)
     end
 end
@@ -275,7 +274,7 @@ function _uninstall_target_shared_libraries(target, batchcmds_, opt)
     -- do uninstall
     for _, libfile in ipairs(libfiles) do
         local filename = path.filename(libfile)
-        batchcmds_:rm(libfile, path.join(bindir, filename), {emptydirs = true})
+        batchcmds_:rm(path.join(bindir, filename), {emptydirs = true})
     end
 end
 
@@ -564,7 +563,7 @@ end
 function _on_installcmd(package, batchcmds_)
     local srcfiles, dstfiles = package:installfiles()
     for idx, srcfile in ipairs(srcfiles) do
-        batchcmds_:cp(srcfile, dstfiles[idx])
+        batchcmds_:cp(srcfile, dstfiles[idx], {symlink = true})
     end
     for _, target in ipairs(package:targets()) do
         _get_target_installcmds(target, batchcmds_, {package = package})
